;*************************************************************** 
;* Feladat: rjon egy programot az AVR Experiment krtyra, amely 
;  egy sszeadt valst meg. Az sszead 4 bites, elojel nlkli
;  binris kdols operandusokkal dolgozik. A szmokat az SW0..3 
;  kapcsolkon kell belltani. Az IT gomb elso megnyomsnak
;  hatsra a szm betltodik a 8 bites eredmnyregiszterbe. Az 
;  eredmnyregiszter tartalmt a LED0..7 jelzi ki. Az IT gomb 
;  msodik megnyomsa beviszi a kapcsolkon belltott msodik
;  operandust, s rgtn hozzadja az eredmnyregiszter 
;  tartalmhoz. Az IT gomb harmadik megnyomsa trli az 
;  eredmnyregisztert s alapllapotba viszi vissza az sszeadt. 
;  A nyomgomb pergsmentestst oldja meg. 
;
;* Rvid lers: sszead ksztse
; 
;* Szerzk: Okos Dniel
;* Mrcsoport: <merocsoport jele>
;
;***************************************************************
;* "AVR ExperimentBoard" port assignment information:
;***************************************************************
;*
;* LED0(P):PortC.0          LED4(P):PortC.4
;* LED1(P):PortC.1          LED5(P):PortC.5
;* LED2(S):PortC.2          LED6(S):PortC.6
;* LED3(Z):PortC.3          LED7(Z):PortC.7        INT:PortE.4
;*
;* SW0:PortG.0     SW1:PortG.1     SW2:PortG.4     SW3:PortG.3
;* 
;* BT0:PortE.5     BT1:PortE.6     BT2:PortE.7     BT3:PortB.7
;*
;***************************************************************
;*
;* AIN:PortF.0     NTK:PortF.1    OPTO:PortF.2     POT:PortF.3
;*
;***************************************************************
;*
;* LCD1(VSS) = GND         LCD9(DB2): -
;* LCD2(VDD) = VCC         LCD10(DB3): -
;* LCD3(VO ) = GND         LCD11(DB4): PortA.4
;* LCD4(RS ) = PortA.0     LCD12(DB5): PortA.5
;* LCD5(R/W) = GND         LCD13(DB6): PortA.6
;* LCD6(E  ) = PortA.1     LCD14(DB7): PortA.7
;* LCD7(DB0) = -           LCD15(BLA): VCC
;* LCD8(DB1) = -           LCD16(BLK): PortB.5 (1=Backlight ON)
;*
;***************************************************************

.include "m128def.inc" ; Definition file for ATmega128 
;* Program Constants 
.equ const =$00 ; Generic Constant Structure example  
;* Program Variables Definitions 
.def temp 	=r16 ; Temporary Register example 
.def res 	=r17 ; sszeads eredmnye
.def state 	=r19 ; Aktulis llapot
.def pressed	=r20 ; Gombnyomst jelzo "flag"
.def pattern 	=r21 ; Bitminta pergsmentestshez

;*************************************************************** 
;* Reset & Interrupt Vectors  
.cseg 
.org $0000 ; Define start of Code segment 
	jmp RESET ; Reset Handler, jmp is 2 word instruction 
	jmp DUMMY_IT	; Ext. INT0 Handler
	jmp DUMMY_IT	; Ext. INT1 Handler
	jmp DUMMY_IT	; Ext. INT2 Handler
	jmp DUMMY_IT	; Ext. INT3 Handler
	jmp DUMMY_IT	; Ext. INT4 Handler (INT gomb)
	jmp DUMMY_IT	; Ext. INT5 Handler
	jmp DUMMY_IT	; Ext. INT6 Handler
	jmp DUMMY_IT	; Ext. INT7 Handler
	jmp DUMMY_IT	; Timer2 Compare Match Handler 
	jmp DUMMY_IT	; Timer2 Overflow Handler 
	jmp DUMMY_IT	; Timer1 Capture Event Handler 
	jmp DUMMY_IT	; Timer1 Compare Match A Handler 
	jmp DUMMY_IT	; Timer1 Compare Match B Handler 
	jmp DUMMY_IT	; Timer1 Overflow Handler 
	jmp TIMER_IT	; Timer0 Compare Match Handler 
	jmp DUMMY_IT	; Timer0 Overflow Handler 
	jmp DUMMY_IT	; SPI Transfer Complete Handler 
	jmp DUMMY_IT	; USART0 RX Complete Handler 
	jmp DUMMY_IT	; USART0 Data Register Empty Hanlder 
	jmp DUMMY_IT	; USART0 TX Complete Handler 
	jmp DUMMY_IT	; ADC Conversion Complete Handler 
	jmp DUMMY_IT	; EEPROM Ready Hanlder 
	jmp DUMMY_IT	; Analog Comparator Handler 
	jmp DUMMY_IT	; Timer1 Compare Match C Handler 
	jmp DUMMY_IT	; Timer3 Capture Event Handler 
	jmp DUMMY_IT	; Timer3 Compare Match A Handler 
	jmp DUMMY_IT	; Timer3 Compare Match B Handler 
	jmp DUMMY_IT	; Timer3 Compare Match C Handler 
	jmp DUMMY_IT	; Timer3 Overflow Handler 
	jmp DUMMY_IT	; USART1 RX Complete Handler 
	jmp DUMMY_IT	; USART1 Data Register Empty Hanlder 
	jmp DUMMY_IT	; USART1 TX Complete Handler 
	jmp DUMMY_IT	; Two-wire Serial Interface Handler 
	jmp DUMMY_IT	; Store Program Memory Ready Handler 

.org $0046

;****************************************************************
;* DUMMY_IT interrupt handler -- CPU hangup with LED pattern
;* (This way unhandled interrupts will be noticed)

;< tbbi IT kezel a fjl vgre! >

DUMMY_IT:	
	ldi r16,   0xFF ; LED pattern:  *-
	out DDRC,  r16  ;               -*
	ldi r16,   0xA5	;               *-
	out PORTC, r16  ;               -*
DUMMY_LOOP:
	rjmp DUMMY_LOOP ; endless loop

;< tbbi IT kezel a fjl vgre! >

;*************************************************************** 
;* MAIN program, Initialisation part
.org $004B;
RESET: 
;* Stack Pointer init, 
;  Set stack pointer to top of RAM 
	ldi temp, LOW(RAMEND) ; RAMEND = "max address in RAM"
	out SPL, temp 	      ; RAMEND value in "m128def.inc" 
	ldi temp, HIGH(RAMEND) 
	out SPH, temp 

M_INIT:
;< ki- s bemenetek inicializlsa stb > 
	; LED init
	ldi temp, 0xFF		; irny - kimenet
	out DDRC, temp
	ldi temp, 0x00		; egyik sem vilgt
	out PORTC, temp

	; SW init
	ldi temp, 0x00		; irny - bemenet
	sts DDRG, temp		; G port -> out helyett sts!!

	; BTN init
	ldi temp, 0x00		; irny - bemenet
	out DDRE, temp

	; TIMER - csak pergsmentestshez
	ldi temp, 0x0f		; 1024-es elooszt -> 10800Hz
	out TCCR0, temp	
	ldi temp, 107		; 108 ciklus -> 10800/108 = 100Hz
	out OCR0, temp
	ldi temp, 0x02		; TIMER IT engedlyezs
	out TIMSK, temp

	ldi state, 0		; kezdollapot
	ldi res, 0		; eredmnyreg. init
	ldi pressed, 0		; gomb nincs lenyomva
	ldi pattern, 0xff	; bitminta kezdetben csupa 1

	sei			; globlis IT engedlyezs

;*************************************************************** 
;* MAIN program, Endless loop part
 
M_LOOP: 

;< fciklus >

	tst pressed		; test for zero
	breq M_LOOP		; ha nincs gombnyoms, ugrunk
	clr pressed		; gombnyoms esetn flag trlse

	cpi state, 1		; 1-es llapot: OP1 betltse
	breq OP1		;  s kijelzs
	cpi state, 2		; 2-es llapot: OP2 betlts
	breq OP2		;  s sszeads, kijelzs
	cpi state, 3		; 3-as llapot: trls, alapll.
	breq CLEAR

	jmp M_LOOP ; Endless Loop  

;*************************************************************** 
;* Subroutines, Interrupt routines

TIMER_IT:			; TIMER IT rutin
	push temp		; ments
	in temp, SREG
	push temp

	; pergsmentests
	in temp, PINE		; gombok beolvassa
	bst temp, 5		; BT0 llapot eltrolsa
	lsl pattern		; bitminta shiftelse balra
	bld pattern, 0		; az LSB helyre beszrjuk 
				; a gomb llapott jelzo bitet

	andi pattern, 0x0f	; felso 4 bitet eldobjuk
	cpi pattern, 0b00001100 ; 1100 bitminta?
	brne NOT_PRESSED	; ha nem, nincs gombnyoms,
				; vagy csak pergst szlel

	; 1100 -> tnyleges lefut l
	inc pressed		; lenyoms trtnt, flag bell.
	inc state		; kvetkezo llapot

NOT_PRESSED:
	pop temp		; visszallts
	out SREG, temp
	pop temp
	reti

OP1:				; elso operandus betltse
	lds res, PING		; kapcsol beolvass, in helyett lds!
	bst res, 4		; SW2 bit thelyezse a helyre
	bld res, 2
	andi res, 0x0f		; felso 4 bit eldobsa
	out PORTC, res		; kijelzs
	jmp M_LOOP

OP2:				; msodik op. betlts + sszeads
	lds temp, PING		; kapcsol beolvass, in helyett lds!
	bst temp, 4		; SW2 bit thelyezse a helyre
	bld temp, 2
	andi temp, 0x0f		; felso 4 bit eldobsa

	add res, temp		; OP1, OP2 sszeads
	out PORTC, res		; kijelzs
	jmp M_LOOP

CLEAR:				; alapllapotba llts
	ldi state, 0		; kezdoll.
	ldi res, 0		; eredmnyreg. trls
	ldi temp, 0		; temp trls
	out PORTC, res		; kijelzs
	jmp M_LOOP